package be.iminds.ilabt.jfed.experimenter_gui.slice;

import be.iminds.ilabt.jfed.espec.model.ESpecStep;
import be.iminds.ilabt.jfed.experiment.*;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEvent;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEventHandler;
import be.iminds.ilabt.jfed.experiment.events.ExperimentEventType;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.google.common.util.concurrent.ListenableFuture;
import javafx.stage.Window;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import javax.inject.Singleton;
import java.time.Instant;
import java.util.Collection;
import java.util.List;

/**
 * Uses jFedGUiConfig to check if certain functionality of ExperimentController needs to be blocked/disabled
 */
@Singleton
public class FunctionalityBlockingExperimentControllerFactory implements ExperimentControllerFactory {
    @Nonnull private final JFedGuiConfig guiConfig;
    @Nonnull private final ExperimentControllerFactory experimentControllerFactory;

    public FunctionalityBlockingExperimentControllerFactory(@Nonnull JFedGuiConfig guiConfig,
                                                            @Nonnull ExperimentControllerFactory experimentControllerFactory) {
        this.guiConfig = guiConfig;
        this.experimentControllerFactory = experimentControllerFactory;
    }

    @Inject
    public FunctionalityBlockingExperimentControllerFactory(@Nonnull JFedGuiConfig guiConfig,
                                                            @Nonnull ExperimentControllerFactoryImpl experimentControllerFactory) {
        this.guiConfig = guiConfig;
        this.experimentControllerFactory = experimentControllerFactory;
    }

    private boolean hasAnyDisabledFunctions() {
        return guiConfig.isExperimentDeleteDisabled() ||
                guiConfig.isExperimentActionDisabled() ||
                guiConfig.isExperimentRenewDisabled() ||
                guiConfig.isExperimentStartDisabled();
    }

    @Override
    public ExperimentController createExperimentController(Experiment experiment) {
        if (hasAnyDisabledFunctions())
            return new LimitedExperimentController(
                    guiConfig,
                    experimentControllerFactory.createExperimentController(experiment)
            );
        else
            return experimentControllerFactory.createExperimentController(experiment);
    }

    private static class LimitedExperimentController implements ExperimentController {
        @Nonnull private final JFedGuiConfig guiConfig;
        @Nonnull private final ExperimentController experimentController;

        public LimitedExperimentController(@Nonnull JFedGuiConfig guiConfig,
                                           @Nonnull ExperimentController experimentController) {
            super();
            this.guiConfig = guiConfig;
            this.experimentController = experimentController;
        }

        @Override
        public void onExperimentStateChange(ExperimentState newExperimentState) {
            experimentController.onExperimentStateChange(newExperimentState);
        }

        @Override
        public void onExperimentPartAdded(ExperimentPart experimentPart) {
            experimentController.onExperimentPartAdded(experimentPart);
        }

        @Override
        public void start() {
            if (guiConfig.isExperimentStartDisabled())
                return;
            experimentController.start();
        }

        @Override
        public ListenableFuture<Void> createSlice() {
            if (guiConfig.isExperimentStartDisabled())
                return null;
            return experimentController.createSlice();
        }

        @Override
        public ListenableFuture<Void> requestUpdate() {
            return experimentController.requestUpdate();
        }

        @Override
        public void shareWithUsers(@Nonnull List<GeniUrn> selectedUsers, boolean registerSshKeys) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.shareWithUsers(selectedUsers, registerSshKeys);
        }

        @Override
        public void unshareWithUsers(@Nonnull List<GeniUrn> selectedUsers, boolean unregisterSshKeys) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.unshareWithUsers(selectedUsers, unregisterSshKeys);
        }

        @Override
        public void updateSshKeys(@Nonnull List<UserSpec> userspec) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.updateSshKeys(userspec);
        }

        @Override
        public void reloadOS(Sliver sliver) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.reloadOS(sliver);
        }

        @Override
        public void createDiskImage(FXRspecNode node, String imageName, boolean global, boolean updatePrepare) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.createDiskImage(node, imageName, global, updatePrepare);
        }

        @Override
        public void stop(Collection<ExperimentPart> experimentParts) {
            if (guiConfig.isExperimentDeleteDisabled())
                return;
            experimentController.stop(experimentParts);
        }

        @Override
        public void stop() {
            if (guiConfig.isExperimentDeleteDisabled())
                return;
            experimentController.stop();
        }

        @Override
        public ListenableFuture<Boolean> testLinks() {
            return experimentController.testLinks();
        }

        @Override
        public void reboot(Sliver sliver) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.reboot(sliver);
        }

        @Override
        public void reboot(ExperimentPart experimentPart) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.reboot(experimentPart);
        }

        @Override
        public void openConsole(Sliver sliver) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.openConsole(sliver);
        }

        @Override
        public void shareLan(FXRspecLink link, String sharedLanName) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.shareLan(link, sharedLanName);
        }

        @Override
        public void unshareLan(FXRspecLink link, String sharedLanName) {
            if (guiConfig.isExperimentActionDisabled())
                return;
            experimentController.unshareLan(link, sharedLanName);
        }

        @Override
        public ListenableFuture<Collection<UserSpec>> fetchSliceMemberSshKeys() {
            return experimentController.fetchSliceMemberSshKeys();
        }

        @Override
        public ListenableFuture<ExperimentTaskStatus> renew(Instant newExpirationTime) {
            if (guiConfig.isExperimentRenewDisabled())
                return null;
            return experimentController.renew(newExpirationTime);
        }

        @Override
        public void renewParts(Instant newExpirationTime, Collection<ExperimentPart> parts) {
            if (guiConfig.isExperimentRenewDisabled())
                return;
            experimentController.renewParts(newExpirationTime, parts);
        }

        @Override
        public void waitForReady() {
            experimentController.waitForReady();
        }

        @Override
        public void setupSoftware() {
            experimentController.setupSoftware();
        }

        @Override
        public void rerunESpec(@Nullable ESpecStep startPosition,
                               @Nullable ESpecStep stopPosition) {
            experimentController.rerunESpec(startPosition, stopPosition);
        }

        @Override
        @Nonnull
        public Experiment getExperiment() {
            return experimentController.getExperiment();
        }

        @Override
        public Window getParentWindow() {
            return experimentController.getParentWindow();
        }

        @Override
        public void setParentWindow(Window parentWindow) {
            experimentController.setParentWindow(parentWindow);
        }

        @Override
        public void addListener(ExperimentControllerListener listener) {
            experimentController.addListener(listener);
        }

        @Override
        public void removeListener(ExperimentControllerListener listener) {
            experimentController.removeListener(listener);
        }

        @Override
        public ExperimentConnectivityTesterFactory getExperimentConnectivityTesterFactory() {
            return experimentController.getExperimentConnectivityTesterFactory();
        }

        @Override
        public void setExperimentConnectivityTesterFactory(ExperimentConnectivityTesterFactory experimentConnectivityTesterFactory) {
            experimentController.setExperimentConnectivityTesterFactory(experimentConnectivityTesterFactory);
        }

        @Override
        public void setExperimentLinkTesterFactory(ExperimentLinkTesterFactory experimentLinkTesterFactory) {
            experimentController.setExperimentLinkTesterFactory(experimentLinkTesterFactory);
        }

        @Override
        public WaitForReadyTimeoutHandler getWaitForReadyTimeoutHandler() {
            return experimentController.getWaitForReadyTimeoutHandler();
        }

        @Override
        public void setWaitForReadyTimeoutHandler(WaitForReadyTimeoutHandler waitForReadyTimeoutHandler) {
            experimentController.setWaitForReadyTimeoutHandler(waitForReadyTimeoutHandler);
        }

        @Override
        public <T extends ExperimentEvent> void addEventHandler(ExperimentEventType<T> type, ExperimentEventHandler<T> eventHandler) {
            experimentController.addEventHandler(type, eventHandler);
        }
    }
}
